function varargout = SliceBrowser5(varargin)
%SLICEBROWSER5 M-file for SliceBrowser5.fig
%       SliceBrowser5 is an interactive viewer of 3D volumes, 
%       it shows 3 perpendicular slices (XY, YZ, ZX) with 3D pointer.
%   Input:  a) VOLUME - a 3D matrix with volume data
%           b) VOLUME - a 4D matrix with volume data over time
%   Control:
%       - Clicking into the window changes the location of 3D pointer.
%       - 3D pointer can be moved also by keyboard arrows.
%       - Pressing +/- will switch to next/previous volume.
%       - Pressing 1,2,3 will change the focus of current axis.
%       - Pressing 'e' will print the location of 3D pointer.
%   Example of usage:
%       load mri.dat
%       volume = squeeze(D);
%       SliceBrowser5(volume);
%
% Author: Marian Uhercik, CMP, CTU in Prague
% Web: http://cmp.felk.cvut.cz/~uhercik/3DSliceViewer/3DSliceViewer.htm
% Last Modified by 05-Aug-2008

% Documentation generated GUIDE:
%
%SLICEBROWSER5 M-file for SliceBrowser5.fig
%      SLICEBROWSER5, by itself, creates a new SLICEBROWSER5 or raises the existing
%      singleton*.
%
%      H = SLICEBROWSER5 returns the handle to a new SLICEBROWSER5 or the handle to
%      the existing singleton*.
%
%      SLICEBROWSER5('Property','Value',...) creates a new SLICEBROWSER5 using the
%      given property value pairs. Unrecognized properties are passed via
%      varargin to SliceBrowser5_OpeningFcn.  This calling syntax produces a
%      warning when there is an existing singleton*.
%
%      SLICEBROWSER5('CALLBACK') and SLICEBROWSER5('CALLBACK',hObject,...) call the
%      local function named CALLBACK in SLICEBROWSER5.M with the given input
%      arguments.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help SliceBrowser5

% Last Modified by GUIDE v2.5 26-Jul-2009 18:17:58

% Begin initialization code - DO NOT EDIT
gui_Singleton = 0;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @SliceBrowser5_OpeningFcn, ...
                   'gui_OutputFcn',  @SliceBrowser5_OutputFcn, ...
                   'gui_LayoutFcn',  [], ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
   gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before SliceBrowser5 is made visible.
function SliceBrowser5_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   unrecognized PropertyName/PropertyValue pairs from the
%            command line (see VARARGIN)

% Choose default command line output for SliceBrowser5
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes SliceBrowser5 wait for user response (see UIRESUME)
% uiwait(handles.figure1);

% if (length(varargin) <=0)
%     error('Input volume has not been specified.');
% end;

ample=varargin{1};
alt = varargin{2};
fons = varargin{3};

% volume = varargin{1};
% volume2 = varargin{2};
% 
% if (ndims(volume) ~= 3 && ndims(volume) ~= 4)
%     error('Input volume must have 3 or 4 dimensions.');
% end;
% if (ndims(volume2) ~= 3 && ndims(volume2) ~= 4)
%     error('Input volume2 must have 3 or 4 dimensions.');
% end;
% 

handles.llavors1 = [];
handles.llavors2 = [];
handles.volume = zeros(ample,alt,fons);
handles.volume2= zeros(ample,alt,fons);
handles.V1 = zeros(ample,alt,fons);
handles.V1segm = zeros(ample,alt,fons);
handles.V2 = zeros(ample,alt,fons);
handles.V2segm = zeros(ample,alt,fons);

handles.VistaOriginal=1;
handles.VistaSegmentat=0;

% set main wnd title
set(gcf, 'Name', 'MRI GUI')

% init 3D pointer
vol_sz = size(handles.volume); 
if (ndims(handles.volume) == 3)
     vol_sz(4) = 1;
end;
pointer3dt = floor(vol_sz/2)+1;
handles.pointer3dt = pointer3dt;
handles.vol_sz = vol_sz;
% 
plot3slices(hObject, handles);

% stores ID of last axis window 
% (0 means that no axis was clicked yet)
handles.last_axis_id = 0;

% Update handles structure
guidata(hObject, handles);


% --- Outputs from this function are returned to the command line.
function varargout = SliceBrowser5_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on mouse press over axes background.
function Volum1_XY_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to Volum1_XY (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% This object contains the XY slice

%disp('Volum1_XY:BtnDown');
pt=get(gca,'currentpoint');
xpos=round(pt(1,2)); ypos=round(pt(1,1));
zpos = handles.pointer3dt(3);
tpos = handles.pointer3dt(4);
handles.pointer3dt = [xpos ypos zpos tpos];
handles.pointer3dt = clipointer3d(handles.pointer3dt,handles.vol_sz);
plot3slices(hObject, handles);
% store this axis as last clicked region
handles.last_axis_id = 1;
% Update handles structure
guidata(hObject, handles);

% --- Executes on mouse press over axes background.
function Volum1_XZ_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to Volum1_XZ (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% This object contains the YZ slice

%disp('Volum1_XZ:BtnDown');
pt=get(gca,'currentpoint');
xpos=round(pt(1,2)); zpos=round(pt(1,1));
ypos = handles.pointer3dt(2);
tpos = handles.pointer3dt(4);
handles.pointer3dt = [xpos ypos zpos tpos];
handles.pointer3dt = clipointer3d(handles.pointer3dt,handles.vol_sz);
plot3slices(hObject, handles);
% store this axis as last clicked region
handles.last_axis_id = 2;
% Update handles structure
guidata(hObject, handles);

% --- Executes on mouse press over axes background.
function Volum1_ZY_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to Volum1_ZY (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% This object contains the XZ slice

%disp('Volum1_ZY:BtnDown');
pt=get(gca,'currentpoint');
zpos=round(pt(1,2)); ypos=round(pt(1,1));
xpos = handles.pointer3dt(1);
tpos = handles.pointer3dt(4);
handles.pointer3dt = [xpos ypos zpos tpos];
handles.pointer3dt = clipointer3d(handles.pointer3dt,handles.vol_sz);
plot3slices(hObject, handles);
% store this axis as last clicked region
handles.last_axis_id = 5;
% Update handles structure
guidata(hObject, handles);

% --- Executes on key press with focus on SliceBrowserFigure and no controls selected.
function SliceBrowserFigure_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to SliceBrowserFigure (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%disp('SliceBrowserFigure_KeyPressFcn');
curr_char = int8(get(gcf,'CurrentCharacter'));
if isempty(curr_char)
    return;
end;

xpos = handles.pointer3dt(1);
ypos = handles.pointer3dt(2);
zpos = handles.pointer3dt(3); 
tpos = handles.pointer3dt(4); 
% Keys:
% - up:   30
% - down:   31
% - left:   28
% - right:   29
% - '1': 49
% - '2': 50
% - '3': 51
% - 'e': 101
% - plus:  43
% - minus:  45
switch curr_char
    case 30
        switch handles.last_axis_id
            case 1
                xpos = xpos -1;
            case 2
                xpos = xpos -1;
            case 3
                zpos = zpos -1;
            case 0
        end;
    case 31
        switch handles.last_axis_id
            case 1
                xpos = xpos +1;
            case 2
                xpos = xpos +1;
            case 3
                zpos = zpos +1;
            case 0
        end;
    case 28
        switch handles.last_axis_id
            case 1
                ypos = ypos -1;
            case 2
                zpos = zpos -1;
            case 3
                ypos = ypos -1;
            case 0
        end;
    case 29
        switch handles.last_axis_id
            case 1
                ypos = ypos +1;
            case 2
                zpos = zpos +1;
            case 3
                ypos = ypos +1;
            case 0
        end;
    case 43
        % plus key
        tpos = tpos+1;
    case 45
        % minus key
        tpos = tpos-1;
    case 49
        % key 1
        handles.last_axis_id = 1;
    case 50
        % key 2
        handles.last_axis_id = 2;
    case 51
        % key 3
        handles.last_axis_id = 3;
    case 101
        disp(['[' num2str(xpos) ' ' num2str(ypos) ' ' num2str(zpos) ' ' num2str(tpos) ']']);
    otherwise
        return
end;
handles.pointer3dt = [xpos ypos zpos tpos];
handles.pointer3dt = clipointer3d(handles.pointer3dt,handles.vol_sz);
plot3slices(hObject, handles);
% Update handles structure
guidata(hObject, handles);

% --- Plots all 3 slices XY, YZ, XZ into 3 subplots
function [sp1,sp2,sp5, sp3, sp4, sp7] = plot3slices(hObject, handles)
% pointer3d     3D coordinates in volume matrix (integers)

handles.pointer3dt;
size(handles.volume);
value3dt = handles.volume(handles.pointer3dt(1), handles.pointer3dt(2), handles.pointer3dt(3), handles.pointer3dt(4));
value3dt_2 = handles.volume2(handles.pointer3dt(1), handles.pointer3dt(2), handles.pointer3dt(3), handles.pointer3dt(4));

text_str = ['[X:' int2str(handles.pointer3dt(1)) ...
           ', Y:' int2str(handles.pointer3dt(2)) ...
           ', Z:' int2str(handles.pointer3dt(3)) ...
           '], V1:' num2str(value3dt), '/ V2: ', num2str(value3dt_2)];
set(handles.pointer3d_info, 'String', text_str);
guidata(hObject, handles);

sliceXY = squeeze(handles.volume(:,:,handles.pointer3dt(3),handles.pointer3dt(4)))+1;
sliceYZ = squeeze(handles.volume(handles.pointer3dt(1),:,:,handles.pointer3dt(4)))+1;
sliceXZ = squeeze(handles.volume(:,handles.pointer3dt(2),:,handles.pointer3dt(4)))+1;

sliceXY_2 = squeeze(handles.volume2(:,:,handles.pointer3dt(3),handles.pointer3dt(4)))+1;
sliceYZ_2 = squeeze(handles.volume2(handles.pointer3dt(1),:,:,handles.pointer3dt(4)))+1;
sliceXZ_2 = squeeze(handles.volume2(:,handles.pointer3dt(2),:,handles.pointer3dt(4)))+1;


subplot(2,2,1,'Parent',handles.panelV1)
%colorbar;
%imagesc(sliceXY);
imagesc(sliceXY);
title('Slice XY');
ylabel('X');xlabel('Y');
line([handles.pointer3dt(2) handles.pointer3dt(2)], [0 size(handles.volume,1)]);
line([0 size(handles.volume,2)], [handles.pointer3dt(1) handles.pointer3dt(1)]);
%set(allchild(gca),'ButtonDownFcn',@Volum1_XY_ButtonDownFcn);
set(allchild(gca),'ButtonDownFcn','SliceBrowser5(''Volum1_XY_ButtonDownFcn'',gca,[],guidata(gcbo))');

sp2 = subplot(2,2,2);
%imagesc(sliceXZ);
imagesc(sliceXZ);
title('Slice XZ');
ylabel('X');xlabel('Z');
line([handles.pointer3dt(3) handles.pointer3dt(3)], [0 size(handles.volume,1)]);
line([0 size(handles.volume,3)], [handles.pointer3dt(1) handles.pointer3dt(1)]);
%set(allchild(gca),'ButtonDownFcn',@Volum1_XZ_ButtonDownFcn);
set(allchild(gca),'ButtonDownFcn','SliceBrowser5(''Volum1_XZ_ButtonDownFcn'',gca,[],guidata(gcbo))');

sp5 = subplot(2,2,3);
%imagesc(sliceYZ');
imagesc(sliceYZ');
title('Slice ZY');
ylabel('Z');xlabel('Y');
line([0 size(handles.volume,2)], [handles.pointer3dt(3) handles.pointer3dt(3)]);
line([handles.pointer3dt(2) handles.pointer3dt(2)], [0 size(handles.volume,3)]);
%set(allchild(gca),'ButtonDownFcn',@Volum1_ZY_ButtonDownFcn);
set(allchild(gca),'ButtonDownFcn','SliceBrowser5(''Volum1_ZY_ButtonDownFcn'',gca,[],guidata(gcbo))');

subplot(2,2,1,'Parent',handles.panelV2)
%colorbar;
%imagesc(sliceXY);
imagesc(sliceXY_2);
title('Slice XY');
ylabel('X');xlabel('Y');
line([handles.pointer3dt(2) handles.pointer3dt(2)], [0 size(handles.volume,1)]);
line([0 size(handles.volume,2)], [handles.pointer3dt(1) handles.pointer3dt(1)]);
%set(allchild(gca),'ButtonDownFcn',@Volum1_XY_ButtonDownFcn);
set(allchild(gca),'ButtonDownFcn','SliceBrowser5(''Volum2_XY_ButtonDownFcn'',gca,[],guidata(gcbo))');

sp4 = subplot(2,2,2);
%imagesc(sliceXZ);
imagesc(sliceXZ_2);
title('Slice XZ');
ylabel('X');xlabel('Z');
line([handles.pointer3dt(3) handles.pointer3dt(3)], [0 size(handles.volume,1)]);
line([0 size(handles.volume,3)], [handles.pointer3dt(1) handles.pointer3dt(1)]);
%set(allchild(gca),'ButtonDownFcn',@Volum1_XZ_ButtonDownFcn);
set(allchild(gca),'ButtonDownFcn','SliceBrowser5(''Volum2_XZ_ButtonDownFcn'',gca,[],guidata(gcbo))');

sp7 = subplot(2,2,3);
%imagesc(sliceYZ');
imagesc(sliceYZ_2');
title('Slice ZY');
ylabel('Z');xlabel('Y');
line([0 size(handles.volume,2)], [handles.pointer3dt(3) handles.pointer3dt(3)]);
line([handles.pointer3dt(2) handles.pointer3dt(2)], [0 size(handles.volume,3)]);
%set(allchild(gca),'ButtonDownFcn',@Volum1_ZY_ButtonDownFcn);
set(allchild(gca),'ButtonDownFcn','SliceBrowser5(''Volum2_ZY_ButtonDownFcn'',gca,[],guidata(gcbo))');





function pointer3d_out = clipointer3d(pointer3d_in,vol_size)
pointer3d_out = pointer3d_in;
for p_id=1:4
    if (pointer3d_in(p_id) > vol_size(p_id))
        pointer3d_out(p_id) = vol_size(p_id);
    end;
    if (pointer3d_in(p_id) < 1)
        pointer3d_out(p_id) = 1;
    end;
end;


% --- Executes on mouse press over axes background.
function Volum2_XY_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to Volum2_XY (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% hObject    handle to Volum1_ZY (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% This object contains the XZ slice

%disp('Volum1_ZY:BtnDown');
pt=get(gca,'currentpoint');
xpos=round(pt(1,2)); ypos=round(pt(1,1));
zpos = handles.pointer3dt(3);
tpos = handles.pointer3dt(4);
handles.pointer3dt = [xpos ypos zpos tpos];
handles.pointer3dt = clipointer3d(handles.pointer3dt,handles.vol_sz);
plot3slices(hObject, handles);
% store this axis as last clicked region
handles.last_axis_id = 1;
% Update handles structure
guidata(hObject, handles);


% --- Executes on mouse press over axes background.
function Volum2_XZ_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to Volum2_XZ (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% hObject    handle to Volum1_ZY (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% This object contains the XZ slice

%disp('Volum1_ZY:BtnDown');
pt=get(gca,'currentpoint');
xpos=round(pt(1,2)); zpos=round(pt(1,1));
ypos = handles.pointer3dt(2);
tpos = handles.pointer3dt(4);
handles.pointer3dt = [xpos ypos zpos tpos];
handles.pointer3dt = clipointer3d(handles.pointer3dt,handles.vol_sz);
plot3slices(hObject, handles);
% store this axis as last clicked region
handles.last_axis_id = 2;
% Update handles structure
guidata(hObject, handles);


% --- Executes on mouse press over axes background.
function Volum2_ZY_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to Volum2_ZY (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% hObject    handle to Volum1_ZY (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% This object contains the XZ slice

%disp('Volum1_ZY:BtnDown');
pt=get(gca,'currentpoint');
zpos=round(pt(1,2)); ypos=round(pt(1,1));
xpos = handles.pointer3dt(1);
tpos = handles.pointer3dt(4);
handles.pointer3dt = [xpos ypos zpos tpos];
handles.pointer3dt = clipointer3d(handles.pointer3dt,handles.vol_sz);
plot3slices(hObject, handles);
% store this axis as last clicked region
handles.last_axis_id = 5;
% Update handles structure
guidata(hObject, handles);




% --- Executes on button press in CarregarV1Button.
function CarregarV1Button_Callback(hObject, eventdata, handles)
% hObject    handle to CarregarV1Button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[V1file,V1path] = uigetfile('*.raws', 'Escull el volum a carregar');

handles.V1 = ReadRawImage([V1path,V1file], 181, 217, 181);
handles.volume = handles.V1;
plot3slices(hObject, handles);
guidata(hObject, handles);


% --- Executes on button press in CarregarV2button.
function CarregarV2button_Callback(hObject, eventdata, handles)
% hObject    handle to CarregarV2button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[V2file,V2path] = uigetfile('*.raws', 'Escull el volum a carregar');

handles.V2 = ReadRawImage([V2path,V2file], 181, 217, 181);
handles.volume2 = handles.V2;
plot3slices(hObject, handles);
guidata(hObject, handles);

% --- Executes on button press in LlavorV1button.
function LlavorV1button_Callback(hObject, eventdata, handles)
% hObject    handle to LlavorV1button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

xpos = handles.pointer3dt(1);
ypos = handles.pointer3dt(2);
zpos = handles.pointer3dt(3);
handles.llavors1 = [handles.llavors1; xpos ypos zpos];
disp(['Afegida la llavor: x=' int2str(xpos) ' y=' int2str(ypos) ' z=' int2str(zpos) ' al V1']);
guidata(hObject, handles);

% --- Executes on button press in LlavorV2button.
function LlavorV2button_Callback(hObject, eventdata, handles)
% hObject    handle to LlavorV2button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

xpos = handles.pointer3dt(1);
ypos = handles.pointer3dt(2);
zpos = handles.pointer3dt(3);
handles.llavors2 = [handles.llavors2; xpos ypos zpos];
disp(['Afegida la llavor: x=' int2str(xpos) ' y=' int2str(ypos) ' z=' int2str(zpos) ' al V2']);
guidata(hObject, handles);

% --- Executes on button press in segmentarbutton.
function segmentarbutton_Callback(hObject, eventdata, handles)
% hObject    handle to segmentarbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[handles.V1segm, nreg1, SIF1, TIF1] = regionGrowing_global(handles.volume, handles.llavors1, 18);
[handles.V2segm, nreg2, SIF2, TIF2] = regionGrowing_global(handles.volume2, handles.llavors2, 18);

handles.volume=handles.V1segm;
handles.volume2=handles.V2segm;

plot3slices(hObject, handles);
set(handles.VistaOriginalbutton, 'Value', 0);
set(handles.VistaSegmentatbutton, 'Value', 1);

handles.VistaOriginal=0;
handles.VistaSegmentat=1;
guidata(hObject, handles);

% --- Executes on button press in VistaOriginalbutton.
function VistaOriginalbutton_Callback(hObject, eventdata, handles)
% hObject    handle to VistaOriginalbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of VistaOriginalbutton
if get(hObject, 'Value')==1
    handles.VistaOriginal=1;
    handles.volume=handles.V1;
    handles.volume2=handles.V2;
    plot3slices(hObject, handles);
end
guidata(hObject, handles);


% --- Executes on button press in VistaSegmentatbutton.
function VistaSegmentatbutton_Callback(hObject, eventdata, handles)
% hObject    handle to VistaSegmentatbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of VistaSegmentatbutton

if get(hObject, 'Value')==1
    handles.VistaSegmentat=1;
    handles.volume=handles.V1segm;
    handles.volume2=handles.V2segm;
    plot3slices(hObject, handles);
end
guidata(hObject, handles);


